unit uBigINIAdditional;

{$mode objfpc}{$H+}

interface

uses
  Classes, Forms, dialogs, SysUtils, FileUtil, ZStream, uBigIni, base64, uTools;


// Defines
function MemoryStreamAsString(vms: TMemoryStream): string;
function GenGUID : string;
procedure iniAttachFile(var ini : TBigIniFile; const fname : String);
function iniExtractAttachment(var ini : TBigIniFile; const fname, output : String) : Boolean;
procedure iniDeleteAttachment(var ini : TBigIniFile; const fname : String);
function iniCompress(const Orig, Fname : String ) : String;
function iniDecompress(const Fname : String) : String;
procedure iniWriteInnerNumberedList(var ini : TBigIniFile; const SectionName, VarName : String; var lst : TStringList);
procedure iniReadInnerNumberedList(var ini : TBigIniFile; const SectionName, VarName : String; var lst : TStringList);





implementation

// Actual

function GenGUID : string;
var
   GID : TGUID;
begin
     CreateGUID(GID);
     result := GUIDToString(GID);
end;

function MemoryStreamAsString(vms: TMemoryStream): string;
  // this will be used to extract the contents of the .bmp
  // as an ordinary string
begin { MemoryStreamAsString }
  SetString(Result, vms.Memory, vms.Size);
end; { MemoryStreamAsString }

procedure iniReadInnerNumberedList(var ini : TBigIniFile; const SectionName, VarName : String; var lst : TStringList);
var
   a, cnt : Integer;

begin
     if not assigned(ini) then exit;
     if (SectionName<>'') and (VarName<>'') then
     begin
         if assigned(lst) then
         begin
             lst.clear;
             cnt := ini.ReadInteger(SectionName, VarName + 'Count', -1);
             if cnt<>-1 then
             begin
                  for a := 0 to cnt - 1 do lst.add(ini.ReadString(SectionName, VarName + 'Line' + IntToStr(a), ''));
             end;
         end;
     end;
end;

procedure iniWriteInnerNumberedList(var ini : TBigIniFile; const SectionName, VarName : String; var lst : TStringList);
var
   a : Integer;
begin
     if not assigned(ini) then exit;
     if (SectionName<>'') and (VarName<>'') then
     begin
         if assigned(lst) then
         begin
             ini.WriteInteger(SectionName, VarName + 'Count', lst.count);
             for a := 0 to lst.count - 1 do ini.WriteString(SectionName, VarName + 'Line' + IntToStr(a), lst[a]);
         end;
     end;
end;

procedure iniDeleteAttachment(var ini : TBigIniFile; const fname : String);
var
   flist : TStringList;
   fStr : String;
begin
     if not Assigned(ini) then exit;
     fStr := ExtractFilename(fname);
     if ini.ReadString('Attachment_' + fStr, 'OriginalFilename', '') <> '' then
     begin
         flist := TStringList.create;
         try
            ini.ReadNumberedList('Attachments', flist, '');
            if flist.indexOf(fSTr)<>-1 then
            begin
                Flist.Delete(flist.IndexOf(FStr));
                ini.EraseSection('Attachment_' + Fstr);
                ini.WriteNumberedList('Attachments', flist);
                ini.FlushFile;
            end;
         finally
                flist.free;
         end;
     end;
end;

function iniExtractAttachment(var ini : TBigIniFile; const fname, output : String) : Boolean;
var
   fstr : String;
   outSTR : String;
   InStr : String;
   lst : TStringList;
begin
     if not assigned(ini) then
     begin
          result := false;
          exit;
     end;
     fstr := Fname;
     inStr := '';
     inStr := trim(ini.ReadString('Attachment_' + FStr, 'Content', ''));
     if FileExists(output) then deleteFile(OUtput);
     outStr := trim(DecodeStringBase64(InStr));
     lst := TStringList.create;
     try
        lst.clear;
        lst.add(trimRight(TrimLeft(OutSTR)));
        if lst.count-1<>-1 then
           if trim(lst[lst.count-1]) = '' then lst.delete(lst.count-1);
        lst.saveToFile(output);
     finally
            lst.free;
     end;
     result := FileExists(output);
end;

procedure iniAttachFile(var ini : TBigIniFile; const fname : String);
var
   fle : TFileStream;
   fList, TLst : TStringList;
   inStr, OutSTR : String;
   Fstr : String;
begin
     if not assigned(ini) then exit;
     try
        fstr := ExtractFileName(Fname);
        fle := TFileStream.create(fname, fmOpenRead or fmSHAREDenYNone);
        try
           fle.position := 0;
           tLST := TStringList.create;
           try
                TLst.LoadFromStream(fle);
                inStr := TLst.text;
                InStr := trim(EncodeStringBase64(inSTR));
                OutStr := trim(InSTR);
                flist := TStringList.create;
                try
                         ini.ReadNumberedList('Attachments', fList, '');
                         if flist.indexOf(fstr)=-1 then
                         begin
                              FList.add(fstr);
                              ini.WriteNumberedList('Attachments', flist);
                         end;
                finally
                          flist.free;
                end;
                ini.WriteString('Attachment_' + FStr, 'OriginalFilename', Fname);
                ini.WriteString('Attachment_' + fSTR, 'ID', GenGUID);
                ini.WriteString('Attachment_' + Fstr, 'Content', Trim(outStr));
                ini.FlushFile;
           finally
                  TLst.free;
           end;
        finally
               fle.free;
        end;
     except
           //
     end;
end;

function iniCompress(const Orig, Fname : String ) : String;
var
   strm: TFileStream;
   comp: Tcompressionstream;
   outs: TFileStream;
   TFname : String;
begin
    if fileExists(fname) then
    begin
        // do compression
        strm := TFileStream.create(fname, fmOpenRead);
        try
           strm.position := 0;
           try
                 TFName := MyGetTempPath + IntToStr(GetTickCount64) + '.tmp';
                 copyFile(Orig, tfname, true);
                 if fileExists(Orig) then deleteFile(Orig);
                 try
                     outs := TFileStream.Create(Orig, fmCreate or fmSHAREdenyNone);
                     try
                          comp := Tcompressionstream.Create(clmax, outs, False);
                          try
                               comp.CopyFrom(strm, strm.Size);
                               comp.flush;
                          finally
                                 comp.Free;
                          end;
                     finally
                            outs.Free;
                     end;
                     sleep(100);
                 except
                       ShowMessage('File may be in use!');
                 end;
           finally
                  strm.free;
           end;
        except
              // Error
        end;
        if fileExists(fname) then deleteFile(Fname);
        if not fileExists(Orig) then
        begin
             ShowMessage('Failed to compress, reverting to back-up!');
             if not CopyFile(Tfname, Orig, true) then showMessage('Failed to revert!');
             Result := Orig;
        end
        else
           result := Orig;
        exit;
    end else
    begin
        ShowMessage('Unable to find file! - Compress');
        result := fname;
        exit;
    end;
end;

function iniDecompress(const Fname : String) : String;
var
   tfname : String;
   strm: TFileStream;
   comp: TDecompressionstream;
   lst : TStringList;
begin
    if fileExists(fname) then
    begin
         // do compression
         tFname := BackSlash(MyGetTempPath) + ExtractFileName(ChangeFileExt(Fname, '')) + IntToStr(GetTickCount64) + '.~ini';
         try
             strm := TFileStream.Create(fname, fmOpenRead or fmSHAREDENYNONE);
             strm.position := 0;
             try
                 comp := Tdecompressionstream.Create(Strm);
                 try
                        if FileExists(TFname) then deleteFile(TFname);
                        comp.Position := 0;
                        lst := TStringList.create;
                        try
                           lst.LoadFromStream(comp);
                           lst.SaveToFile(TFname);
                        finally
                               lst.free;
                        end;
                 finally
                   comp.Free;
                 end;
             finally
                    strm.Free;
             end;
         except
           // ignore
         end;
         result := TFname;
    end else
    begin
        ShowMessage('Unable to find file! - Save');
        result := '';
    end;
end;



end.

